# line protocol

# Method to create a full InfluxDB request statement (based on partial statement from client)
import uuid
from random import random, randint


# Reports TX and RX, scaling on requested quality
def generate_network_report(recieved_bytes, sent_bytes, time):
    # Measurement
    result = 'net_port_io'
    # Tags
    result += ',port_id=enps03 '
    # Fields
    result += 'RX_BYTES_PORT_M=' + str(recieved_bytes) + ","
    result += 'TX_BYTES_PORT_M=' + str(sent_bytes)
    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    # print('network'+result)
    return result


# Formats VM config
def generate_vm_config(state, cpu, mem, storage, time):
    # metric
    result = 'vm_res_alloc'
    # Tags
    result += ',vm_state=' + quote_wrap(state)
    result += ' '
    # Fields
    result += 'cpu=' + str(cpu)
    result += ',memory=' + quote_wrap(mem)
    result += ',storage=' + quote_wrap(storage)

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result


# Reports cpu usage, scaling on requests
def generate_cpu_report(loc, sfc, sfc_i, sf_package, sf_i, cpu_useage, cpu_usage_system, time):
    result = 'vm_host_cpu_usage'
    # Tag
    result += ',location='+quote_wrap(loc)
    result += ',sfc='+quote_wrap(sfc)
    result += ',sfc_i='+quote_wrap(sfc_i)
    result += ',sf_package='+quote_wrap(sf_package)
    result += ',sf_i='+quote_wrap(sf_i)
    result += ' '
    # field
    result += 'cpu_usage='+str(cpu_useage)
    result += ',cpu_usage_system='+str(cpu_usage_system)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Reports response times, scaling on number of requests
def generate_mpegdash_report(resource, requests, avg_response_time, peak_response_time, time):
    # Measurement
    result = 'mpegdash_service '
    # Tags
    # None
    # Fields
    result += 'cont_nav=\"' + str(resource) + "\","
    # result += 'cont_rep=' + str(quality) + ','
    result += 'requests=' + str(requests) + ','
    result += 'avg_response_time=' + str(avg_response_time) + ','
    result += 'peak_response_time=' + str(peak_response_time)
    # Timestamp
    result += ' ' + str(_getNSTime(time))
    # print(result)
    return result


# Influx needs strings to be quoted, this provides a utility interface to do this
def quote_wrap(str):
    return "\"" + str + "\""


# InfluxDB likes to have time-stamps in nanoseconds
def _getNSTime(time):
    # Convert to nano-seconds
    return 1000000 * time

# DEPRICATED
# ____________________________________________________________________________

# DEPRICATED: old structure, not part of new spec
def _generateClientRequest(cReq, id, time):
    # Tags first
    result = 'sid="' + str(id) + '",' + cReq

    # Fields
    # No additional fields here yet

    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    return 'request,' + result


# Method to create a full InfluxDB response statement
# DEPRECATED: old structure, not part of new spec
def _generateServerResponse(reqID, quality, time, cpuUsage, qualityDifference):
    # Tags first
    result = ' '

    # Fields
    result += 'quality=' + str(quality) + ','
    result += 'cpuUsage=' + str(cpuUsage) + ','
    result += 'qualityDifference=' + str(qualityDifference) + ','
    result += 'requestID="' + str(reqID) + '",'
    result += 'index="' + str(uuid.uuid4()) + '"'

    # Timestamp
    result += ' ' + str(_getNSTime(time))

    # Measurement
    # print('response'+result)
    return 'response' + result



# Formats server config
def _generateServerConfig(ID, location, cpu, mem, storage, time):
    # metric
    result = 'host_resource'
    # Tags
    result += ',slice_id=' + quote_wrap(ID)
    result += ',location=' + quote_wrap(location)
    result += ' '
    # Fields
    result += 'cpu=' + str(cpu)
    result += ',memory=' + quote_wrap(mem)
    result += ',storage=' + quote_wrap(storage)

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result



# Format port config
def _configure_port(port_id, state, rate, time):
    # metric
    result = 'net_port_config '
    # Fields
    result += 'port_id=' + quote_wrap('enps' + port_id)
    result += ',port_state=' + quote_wrap(state)
    result += ',tx_constraint=' + quote_wrap(rate)
    result += ' '

    # Time
    result += ' ' + str(_getNSTime(time))

    print(result)
    return result


# Format service function config
def _configure_service_function(state, max_connected_clients):
    # measurement
    result = 'mpegdash_service_config'
    # tags
    result += ',service_state='+quote_wrap(state)
    result += ' '
    # fields
    result += 'max_connected_clients='+str(max_connected_clients)

    return result



# Reports memory usage, scaling on requests
def generate_mem_report(requests, total_mem, time):
    # Measurement
    result = 'mem'
    result += ' '
    # field
    used = randint(0, min(100,5*requests))
    available = 100-used
    result += 'available_percent='+str(available)
    result += ',used_percent='+str(used)
    result += ',total='+str(total_mem)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Formats compute node config
def generate_compute_node_config(slice_id, location, node_id, cpus, mem, storage, time):
    # Measurement
    result = 'compute_node_config'
    # CommonContext Tag
    result += ',slide_id='+quote_wrap(slice_id)
    # Tag
    result += ',location='+quote_wrap(location)
    result += ',comp_node_id='+quote_wrap(node_id)
    result += ' '
    # field
    result += 'cpus='+str(cpus)
    result += ',memory='+str(mem)
    result += ',storage='+str(storage)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Formats network resource config
def generate_network_resource_config(slice_id, network_id, bandwidth, time):
    # Measurement
    result = 'network_resource_config'
    # Meta Tag
    result += ',slice_id='+quote_wrap(slice_id)
    # Tag
    result += 'network_id='+quote_wrap(network_id)
    result += ' '
    # field
    result += 'bandwidth='+str(bandwidth)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Formats network interface config
def generate_network_interface_config(slice_id, comp_node_id, port_id, rx_constraint, tx_constraint, time):
    # Measurement
    result = 'network_interface_config'
    # Meta Tag
    result += ',slice_id'+quote_wrap(slice_id)
    # Tags
    result += ',comp_node_id='+quote_wrap(comp_node_id)
    result += ',port_id='+quote_wrap(port_id)
    result += ' '
    # field
    result += 'rx_constraint='+str(rx_constraint)
    result += ',tx_constraint='+str(tx_constraint)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Format SF instance config
def generate_sf_instance_surrogate_config(loc, sfc, sfc_i, sf_package, sf_i, cpus, mem, storage, time):
    # Measurement
    result = 'sf_instance_surrogate_config'
    # Meta Tag
    result += ',location'+quote_wrap(loc)
    result += ',sfc'+quote_wrap(sfc)
    result += ',sfc_i'+quote_wrap(sfc_i)
    result += ',sf_package'+quote_wrap(sf_package)
    result += ',sf_i'+quote_wrap(sf_i)
    result += ' '
    # field
    result += 'cpus='+str(cpus)
    result += ',memory='+str(mem)
    result += ',storage='+str(storage)
    result += ' '
    # Time
    result += str(_getNSTime(time))
    print(result)
    return result


# Formats context container as part of other line protocol generators
def service_function_measurement(measurement, service_function_context):
    result = measurement
    result += ',sfc'+quote_wrap(service_function_context.sfc)
    result += ',sfc_i'+quote_wrap(service_function_context.sfc_i)
    result += ',sf_package'+quote_wrap(service_function_context.sf_package)
    result += ',sf_i'+quote_wrap(service_function_context.sf_i)

    return result



